@@ -3,11 +3,12 @@ from _tree import (
     tree2array, tree2rec,
     array2tree, array2root,
     list_trees, list_branches, list_structures)
-from _hist import fill_hist
+from _hist import fill_hist, fill_profile
 from _graph import fill_graph
 from _sample import random_sample
 from _array import array
 from _matrix import matrix
+from _evaluate import evaluate
 from _warnings import RootNumpyWarning, RootNumpyUnconvertibleWarning
 from _utils import (
     stretch, blockwise_inner_join,
@@ -22,10 +23,12 @@ __all__ = [
     'array2tree',
     'array2root',
     'fill_hist',
+    'fill_profile',
     'fill_graph',
     'random_sample',
     'array',
     'matrix',
+    'evaluate',
     'list_trees',
     'list_branches',
     'list_structures',
