@@ -11,14 +11,16 @@ import ROOT
 from ROOT import (
     TChain, TFile, TTree,
     TH1D, TH2D, TH3D,
+    TProfile, TProfile2D, TProfile3D,
     TGraph, TGraph2D,
-    TF1, TF2, TF3)
+    TF1, TF2, TF3, TFormula)
 
 import root_numpy as rnp
 from root_numpy.testdata import get_filepath, get_file
 from root_numpy.extern.ordereddict import OrderedDict
 
-from nose.tools import raises, assert_raises, assert_equal, assert_almost_equal
+from nose.tools import (raises, assert_raises, assert_true,
+                        assert_equal, assert_almost_equal)
 
 
 ROOT.gErrorIgnoreLevel = ROOT.kFatal
@@ -229,40 +231,78 @@ def test_specific_branch():
 
 def test_vector():
     a = rnp.root2rec(load('hvector.root'))
-    assert_equal(
-        a.dtype,
-        [('v_i', 'O'),
-         ('v_f', 'O'),
-         ('v_F', 'O'),
-         ('v_d', 'O'),
-         ('v_l', 'O'),
-         ('v_c', 'O'),
-         ('v_b', 'O')])
+    types = [('v_i', 'O'),\
+             ('v_f', 'O'),\
+             ('v_F', 'O'),\
+             ('v_d', 'O'),\
+             ('v_l', 'O'),\
+             ('v_c', 'O'),\
+             ('v_b', 'O'),\
+             ('vv_i', 'O'),\
+             ('vv_f', 'O'),\
+             ('vv_F', 'O'),\
+             ('vv_d', 'O'),\
+             ('vv_l', 'O'),\
+             ('vv_c', 'O')]
+    assert_equal(a.dtype, types )
 
-    assert_equal(a.v_i[1].dtype, np.int32)
-    assert_equal(a.v_f[1].dtype, np.float32)
-    assert_equal(a.v_F[1].dtype, np.float32)
-    assert_equal(a.v_d[1].dtype, np.float64)
-    assert_equal(a.v_l[1].dtype, np.int64)
-    assert_equal(a.v_c[1].dtype, np.int8)
-    assert_equal(a.v_b[1].dtype, np.bool)
+    assert_equal(a.v_i[0].dtype, np.int32)
+    assert_equal(a.v_f[0].dtype, np.float32)
+    assert_equal(a.v_F[0].dtype, np.float32)
+    assert_equal(a.v_d[0].dtype, np.float64)
+    assert_equal(a.v_l[0].dtype, np.int64)
+    assert_equal(a.v_c[0].dtype, np.int8)
+    assert_equal(a.v_b[0].dtype, np.bool)
+
+    # assert that wrapper array is np.object
+    assert_equal(a.vv_i[0].dtype, np.object)
+    assert_equal(a.vv_f[0].dtype, np.object)
+    assert_equal(a.vv_F[0].dtype, np.object)
+    assert_equal(a.vv_d[0].dtype, np.object)
+    assert_equal(a.vv_l[0].dtype, np.object)
+    assert_equal(a.vv_c[0].dtype, np.object)
+
+    assert_equal(a.vv_i[0][0].dtype, np.int32)
+    assert_equal(a.vv_f[0][0].dtype, np.float32)
+    assert_equal(a.vv_F[0][0].dtype, np.float32)
+    assert_equal(a.vv_d[0][0].dtype, np.float64)
+    assert_equal(a.vv_l[0][0].dtype, np.int64)
+    assert_equal(a.vv_c[0][0].dtype, np.int8)
 
     #check couple value
-    assert_equal(a.v_i[1][0], 1)
-    assert_equal(a.v_i[2][1], 3)
-    assert_equal(a.v_i[-1][0], 99)
-    assert_equal(a.v_i[-1][-1], 107)
+    assert_equal(a.v_i[0][0], 1)
+    assert_equal(a.v_i[1][1], 3)
+    assert_equal(a.v_i[-2][0], 99)
+    assert_equal(a.v_i[-2][-1], 107)
 
-    assert_equal(a.v_f[1][0], 2.0)
-    assert_equal(a.v_f[2][1], 5.0)
-    assert_equal(a.v_f[-1][0], 198.0)
-    assert_equal(a.v_f[-1][-1], 206.0)
+    assert_equal(a.v_f[0][0], 2.0)
+    assert_equal(a.v_f[1][1], 5.0)
+    assert_equal(a.v_f[-2][0], 198.0)
+    assert_equal(a.v_f[-2][-1], 206.0)
 
-    assert_equal(a.v_F[1][0], 2.0)
-    assert_equal(a.v_F[2][1], 5.0)
-    assert_equal(a.v_F[-1][0], 198.0)
-    assert_equal(a.v_F[-1][-1], 206.0)
+    assert_equal(a.v_F[0][0], 2.0)
+    assert_equal(a.v_F[1][1], 5.0)
+    assert_equal(a.v_F[-2][0], 198.0)
+    assert_equal(a.v_F[-2][-1], 206.0)
 
+    # stricter conditioning for numpy arrays
+    def assert_equal_array(arr1, arr2):
+      return assert_equal( (arr1 == arr2).all(), True)
+
+    assert_equal_array(a.vv_i[0][0], np.array([1], dtype=np.int32) )
+    assert_equal_array(a.vv_i[1][1], np.array([2, 3], dtype=np.int32) )
+    assert_equal_array(a.vv_i[-2][0], np.array([99], dtype=np.int32) )
+    assert_equal_array(a.vv_i[-2][-1], np.array([ 99, 100, 101, 102, 103, 104, 105, 106, 107], dtype=np.int32) )
+
+    assert_equal_array(a.vv_f[0][0], np.array([ 2.], dtype=np.float32) )
+    assert_equal_array(a.vv_f[1][1], np.array([ 4.,  5.], dtype=np.float32) )
+    assert_equal_array(a.vv_f[-2][0], np.array([ 198.], dtype=np.float32) )
+    assert_equal_array(a.vv_f[-2][-1], np.array([ 198.,  199.,  200.,  201.,  202.,  203.,  204.,  205.,  206.], dtype=np.float32) )
+
+    assert_equal_array(a.vv_F[0][0], np.array([ 2.], dtype=np.float32) )
+    assert_equal_array(a.vv_F[1][1], np.array([ 4.,  5.], dtype=np.float32) )
+    assert_equal_array(a.vv_F[-2][0], np.array([ 198.], dtype=np.float32) )
+    assert_equal_array(a.vv_F[-2][-1], np.array([ 198.,  199.,  200.,  201.,  202.,  203.,  204.,  205.,  206.], dtype=np.float32) )
 
 def test_slice():
     a = rnp.root2rec(load('single1.root'), stop=10)
@@ -342,6 +382,50 @@ def test_fill_hist():
     assert_raises(TypeError, rnp.fill_hist, h, a)
 
 
+def test_fill_profile():
+    np.random.seed(0)
+    w1D = np.empty(1E6)
+    w1D.fill(2.)
+    data1D = np.random.randn(1E6, 2)
+    data2D = np.random.randn(1E6, 3)
+    data3D = np.random.randn(1E4, 4)
+
+    a = TProfile('th1d', 'test', 1000, -5, 5)
+    rnp.fill_profile(a, data1D)
+    assert_true(a.Integral() !=0)
+
+    a_w = TProfile('th1dw', 'test', 1000, -5, 5)
+    rnp.fill_profile(a_w, data1D, w1D)
+    assert_true(a_w.Integral() != 0)
+    assert_equal(a_w.Integral(), a.Integral())
+
+    b = TProfile2D('th2d', 'test', 100, -5, 5, 100, -5, 5)
+    rnp.fill_profile(b, data2D)
+    assert_true(b.Integral() != 0)
+
+    c = TProfile3D('th3d', 'test', 10, -5, 5, 10, -5, 5, 10, -5, 5)
+    rnp.fill_profile(c, data3D)
+    assert_true(c.Integral() != 0)
+
+    # array and weights lengths do not match
+    assert_raises(ValueError, rnp.fill_profile, c, data3D, np.ones(10))
+
+    # weights is not 1D
+    assert_raises(ValueError, rnp.fill_profile, c, data3D,
+                  np.ones((data3D.shape[0], 1)))
+
+    # array is not 2D
+    assert_raises(ValueError, rnp.fill_profile, c, np.ones(10))
+
+    # length of second axis is not one more than dimensionality of the profile
+    for h in (a, b, c):
+        assert_raises(ValueError, rnp.fill_profile, h, np.random.randn(1E4, 5))
+
+    # wrong type
+    assert_raises(TypeError, rnp.fill_profile,
+                  TH1D("test", "test", 1, 0, 1), data1D)
+
+
 def test_fill_graph():
     np.random.seed(0)
     data2D = np.random.randn(1E6, 2)
@@ -434,6 +518,10 @@ def test_blockwise_inner_join():
     a1 = rnp.blockwise_inner_join(
         test_data, ['sl', 'al'], test_data['fk'], ['ar'])
 
+    # specify fk with string
+    a1 = rnp.blockwise_inner_join(
+        test_data, ['sl', 'al'], 'fk', ['ar'])
+
     exp1 = np.array([
         (1.0, 11, 2, 1),
         (1.0, 12, 1, 0),
@@ -443,7 +531,7 @@ def test_blockwise_inner_join():
             ('sl', '<f8'),
             ('al', '<i8'),
             ('ar', '<i8'),
-            ('fk1', '<i8')])
+            ('fk', '<i8')])
     assert_array_equal(a1, exp1, verbose=True)
 
     # vector join with force repeat
@@ -458,7 +546,7 @@ def test_blockwise_inner_join():
             ('sl', '<f8'),
             ('al', '|O8'),
             ('ar', '<i8'),
-            ('fk1', '<i8')])
+            ('fk', '<i8')])
     assert_equal(str(a2), str(exp2)) # numpy testing doesn't like subarray
     assert_equal(a2.dtype, exp2.dtype)
 
@@ -472,7 +560,7 @@ def test_blockwise_inner_join():
             ('sl', '<f8'),
             ('al', '|O8'),
             ('ar', '<i8'),
-            ('fk1', '<i8')])
+            ('fk', '<i8')])
     assert_equal(str(a3), str(exp3)) # numpy testing doesn't like subarray
     assert_equal(a3.dtype, exp3.dtype)
 
@@ -650,3 +738,70 @@ def test_stack():
 def test_dup_idx():
     a = [1, 2, 3, 4, 3, 2]
     assert_array_equal(rnp.dup_idx(a), [1, 2, 4, 5])
+
+
+def test_evaluate():
+    # create functions and histograms
+    f1 = TF1("f1", "x")
+    f2 = TF2("f2", "x*y")
+    f3 = TF3("f3", "x*y*z")
+    h1 = TH1D("h1", "", 10, 0, 1)
+    h1.FillRandom("f1")
+    h2 = TH2D("h2", "", 10, 0, 1, 10, 0, 1)
+    h2.FillRandom("f2")
+    h3 = TH3D("h3", "", 10, 0, 1, 10, 0, 1, 10, 0, 1)
+    h3.FillRandom("f3")
+    # generate random arrays
+    arr_1d = np.random.rand(5)
+    arr_2d = np.random.rand(5, 2)
+    arr_3d = np.random.rand(5, 3)
+    arr_4d = np.random.rand(5, 4)
+    # evaluate the functions
+    assert_array_equal(rnp.evaluate(f1, arr_1d), map(f1.Eval, arr_1d))
+    assert_array_equal(rnp.evaluate(f1.GetTitle(), arr_1d),
+                       map(f1.Eval, arr_1d))
+    assert_array_equal(rnp.evaluate(f2, arr_2d),
+                       [f2.Eval(*x) for x in arr_2d])
+    assert_array_equal(rnp.evaluate(f2.GetTitle(), arr_2d),
+                       [f2.Eval(*x) for x in arr_2d])
+    assert_array_equal(rnp.evaluate(f3, arr_3d),
+                       [f3.Eval(*x) for x in arr_3d])
+    assert_array_equal(rnp.evaluate(f3.GetTitle(), arr_3d),
+                       [f3.Eval(*x) for x in arr_3d])
+    # 4d formula
+    f4 = TFormula('test', 'x*y+z*t')
+    assert_array_equal(rnp.evaluate(f4, arr_4d),
+                       [f4.Eval(*x) for x in arr_4d])
+    # evaluate the histograms
+    assert_array_equal(rnp.evaluate(h1, arr_1d),
+                       [h1.GetBinContent(h1.FindBin(x)) for x in arr_1d])
+    assert_array_equal(rnp.evaluate(h2, arr_2d),
+                       [h2.GetBinContent(h2.FindBin(*x)) for x in arr_2d])
+    assert_array_equal(rnp.evaluate(h3, arr_3d),
+                       [h3.GetBinContent(h3.FindBin(*x)) for x in arr_3d])
+    # create a graph
+    g = TGraph(2)
+    g.SetPoint(0, 0, 1)
+    g.SetPoint(1, 1, 2)
+    assert_array_equal(rnp.evaluate(g, [0, .5, 1]), [1, 1.5, 2])
+    from ROOT import TSpline3
+    s = TSpline3("spline", g)
+    assert_array_equal(rnp.evaluate(s, [0, .5, 1]), map(s.Eval, [0, .5, 1]))
+    # test exceptions
+    assert_raises(TypeError, rnp.evaluate, object(), [1, 2, 3])
+    assert_raises(ValueError, rnp.evaluate, h1, arr_2d)
+    assert_raises(ValueError, rnp.evaluate, h2, arr_3d)
+    assert_raises(ValueError, rnp.evaluate, h2, arr_1d)
+    assert_raises(ValueError, rnp.evaluate, h3, arr_1d)
+    assert_raises(ValueError, rnp.evaluate, h3, arr_2d)
+    assert_raises(ValueError, rnp.evaluate, f1, arr_2d)
+    assert_raises(ValueError, rnp.evaluate, f2, arr_3d)
+    assert_raises(ValueError, rnp.evaluate, f2, arr_1d)
+    assert_raises(ValueError, rnp.evaluate, f3, arr_1d)
+    assert_raises(ValueError, rnp.evaluate, f3, arr_2d)
+    assert_raises(ValueError, rnp.evaluate, g, arr_2d)
+    assert_raises(ValueError, rnp.evaluate, s, arr_2d)
+    assert_raises(ValueError, rnp.evaluate, "f", arr_1d)
+    assert_raises(ValueError, rnp.evaluate, "x*y", arr_1d)
+    assert_raises(ValueError, rnp.evaluate, "x", arr_2d)
+    assert_raises(ValueError, rnp.evaluate, "x*y", arr_3d)
